==> react-native-hotqa-sdk-0.1.9.tgz: package/src/pages/start/view/index.js <==
import React, {useState, useRef, useEffect} from 'react';
import {TouchableOpacity, View, Text, Image, Dimensions} from 'react-native';
import Swiper from '../../../components/swiper';
import Icon from 'react-native-vector-icons/Ionicons';
import {createSelector} from 'reselect';
import {useDispatch, useSelector} from 'react-redux';
import Slider from '@react-native-community/slider';

import SingleChoiceAudio from '../../../components/single-choice-audio';
import SingleChoice from '../../../components/single-choice';
import MultipleChoice from '../../../components/multiple-choice';
import ImageChoice from '../../../components/image-choice';
import Writing from '../../../components/writing';
import AudioRecording from '../../../components/audio-recording';
import Reorder from '../../../components/reorder/index2';
import Matching from '../../../components/matching';
import FillInTheBlank from '../../../components/fill-in-the-blank';
import FillInTheBlankWiring from '../../../components/fill-in-the-blank-writing';

import * as TYPE from '../../../common/constant';
import {onConfigurations} from '../../../common/helper';
import HTMLView from 'react-native-htmlview';
import {styles} from './style';
import ic_play from '../../../assets/images/ic_play.png';
import ic_pause from '../../../assets/images/ic_pause.png';
import song_nhac from '../../../assets/images/song_nhac.png';
import Sound from 'react-native-sound';
import {color_blue} from '../../../assets/colors';
import Player from '../../../components/player';
import {REQUEST_ANSWER_FROM_RESULT} from '../../../redux/actions';

const width = Dimensions.get('screen').width;

Icon.loadFont();
const getData = createSelector(
  (state) => state.reducersAccepted,
  (reducersAccepted) => reducersAccepted,
);

const getKey = createSelector(
  (state) => state.reducerComponents,
  (reducerComponents) => reducerComponents,
);
const getUserAnswer = createSelector(
  (state) => state.submitData,
  (submitData) => submitData,
);
export default (props) => {
  const {navigation, route} = props;
  const dispatch = useDispatch();
  // const player = useRef(null);
  // const [sound, setSound] = useState(null);
  // const [isPlay, setIsPlay] = useState(true);
  const [isChoose, setChoose] = useState(false);
  // const [muted, setMute] = useState(false);
  const {data, loading, error} = useSelector(getData);
  const {keyPage, isFinish, isResult} = useSelector(getKey);
  const {user_answer: answers} = useSelector(getUserAnswer);

  const arrAnswer = [];

  const {sections} = data;
  const sectionsData = sections.map((e, index) => {
    const section_id = e.id;
    const section_tag = e.tag;
    const questions = e.questions || e.answers;
    const outline_s = e.outline;
    const extra = e.extra;
    const titleSection = e.title;
    const questionsData = questions.map((item) => {
      if (isResult) {
        item.id = item.question_id;
        item.title = item.question_title;
        item.type = item.question_type;
        item.configurations = item.question_configuration;
        item.extra = item.question_extra;
        item.tag = item.section_tag;

        const parseAnswer = JSON.parse(item.user_answer);
        if (parseAnswer.value !== '') {
          const itemAnswer = {
            question_id: item.question_id,
            type: item.question_type,
            user_answer: JSON.parse(item.user_answer),
            question_configuration: item.question_configuration,
          };
          arrAnswer.push(itemAnswer);
        }
      } else {
        const itemAnswer = {
          section_id: section_id,
          question_id: item.id,
          type: item.type,
          user_answer: {value: ''},
          tag: section_tag,
          question_configuration: JSON.parse(item.configurations),
        };
        arrAnswer.push(itemAnswer);
      }
      let configurations_type = '';
      const configurations = JSON.parse(item.configurations);
      if (Array.isArray(configurations) && configurations.length > 0) {
        configurations_type = configurations[0].data.type;
      }
      if (item.extra && JSON.parse(item.extra).length > 0) {
        return {
          ...item,
          section_id,
          section_tag,
          outline_s,
          configurations_type,
          titleSection,
        };
      }
      return {
        ...item,
        section_id,
        section_tag,
        outline_s,
        extra,
        configurations_type,
        titleSection,
      };
    });
    return questionsData;
  });

  let limitMedia = 3;
  if (data.extra) {
    const parseExtra = JSON.parse(data.extra);
    if (parseExtra.media) {
      const {limit_play} = parseExtra.media;
      if (limit_play > 0) {
        limitMedia = limit_play;
      }
    }
  }
  const [limitPlay, setLimitPlay] = useState(limitMedia);
  // console.log(63, test_data.extra);

  const questions = sectionsData.flat();

  let sound = null;
  let user_answer = answers;
  if (isResult) {
    user_answer = arrAnswer;
  }

  useEffect(() => {
    dispatch({type: REQUEST_ANSWER_FROM_RESULT, payload: {data: arrAnswer}});
  }, []);

  const onPauseAudio = (type) => {
    if (sound) {
      if (type === 'Record') {
        sound.stop();
      }
    }
  };

  Sound.setCategory('Playback', false);
  const playAudio = (item, index) => {
    if (keyPage === index && limitPlay > 0) {
      if (!sound) {
        sound = new Sound(item.value, null, (error) => {
          if (error) {
            return;
          }
          if (!isChoose) {
            sound.play(() => setLimitPlay(limitPlay - 1));
          }
        });
      }
    }
  };

  const stopAudio = () => {
    if (sound) {
      sound.stop();
    }
  };

  const viewTitle = (item, i) => {
    switch (item.type) {
      case 'image':
        return <Image source={{uri: item.value}} style={styles.imgQuestion} />;
      case 'text':
        // return <Text style={styles.outline}>{item.value}</Text>;
        return <HTMLView value={item.value} style={{marginBottom: 10}} />;
      case 'audio':
        playAudio(item, i);
        return (
          // <Player
          //   keyPage={keyPage}
          //   item={item}
          //   index={i}
          //   sound={sound}
          //   ref={player}
          // />
          <View style={styles.viewPlayer}>
            <TouchableOpacity
              onPress={() => {
                if (sound) {
                  // setChoose(false);
                  sound.play(() => setLimitPlay(limitPlay - 1));
                }
              }}>
              <Image source={ic_play} style={styles.iconPlayPause} />
            </TouchableOpacity>
            <Image
              source={song_nhac}
              style={{flex: 1, height: 50, opacity: 0.3}}
            />
          </View>
        );
    }
  };

  return (
    <View>
      <Swiper
        questions={questions}
        navigation={navigation}
        stopSound={() => {
          setChoose(false);
          setLimitPlay(limitMedia);
          stopAudio();
        }}
        keyPage={keyPage}
        route={route.params?.nav}>
        {questions.map((item, i) => {
          const extra =
            JSON.parse(item.extra) === null ? [] : JSON.parse(item.extra);
          const configurations = JSON.parse(item.configurations);

          const type = item.type;
          const {outline, outline_s} = item;
          // console.log(166, i, item);
          return (
            <View style={[styles.slide]} key={i}>
              {(outline_s ||
                outline ||
                (extra !== null && extra.length > 0)) && (
                <View style={styles.viewOutline}>
                  {outline_s ? (
                    <HTMLView
                      value={outline_s}
                      style={{marginHorizontal: 20}}
                    />
                  ) : null}
                  {outline ? (
                    <HTMLView value={outline} style={{marginHorizontal: 20}} />
                  ) : null}
                  {extra.length > 0 && (
                    <View style={styles.viewTitle}>
                      {/* {viewTitle(extra)} */}
                      {extra[0] && viewTitle(extra[0], i)}
                      {extra[1] && viewTitle(extra[1], i)}
                    </View>
                  )}
                </View>
              )}
              <View style={{flex: 0.6}}>
                <Text style={styles.title}>{item.title}</Text>
                <View style={{marginTop: 10}}>
                  {type === TYPE.SINGLE_CHOICE &&
                    item.configurations_type !== 'image' && (
                      <SingleChoice
                        data={configurations}
                        item_question={item}
                        user_answer={user_answer}
                        onPress={(items) => {
                          setChoose(true);
                          stopAudio();
                          onConfigurations(
                            {...item, items},
                            user_answer,
                            dispatch,
                          );
                        }}
                        keyPage={keyPage}
                        index={i}
                        isFinish={isFinish}
                      />
                    )}

                  {type === TYPE.MULTIPLE_CHOICE &&
                    item.configurations_type !== 'image' && (
                      <MultipleChoice
                        onPress={(items) => {
                          setChoose(true);
                          stopAudio();
                          onConfigurations(
                            {...item, items},
                            user_answer,
                            dispatch,
                          );
                        }}
                        data={configurations}
                        item_question={item}
                        user_answer={user_answer}
                        keyPage={keyPage}
                        index={i}
                        isFinish={isFinish}
                      />
                    )}

                  {item.configurations_type === 'image' && (
                    <ImageChoice
                      data={configurations}
                      type={type}
                      dispatch={dispatch}
                      user_answer={user_answer}
                      item_question={item}
                      keyPage={keyPage}
                      index={i}
                      isFinish={isFinish}
                      audio={() => {
                        setChoose(true);
                        stopAudio();
                      }}
                    />
                  )}
                  {type === TYPE.WRITING && (
                    <Writing
                      item={item}
                      user_answer={user_answer}
                      dispatch={dispatch}
                      isFinish={isFinish}
                      audio={() => {
                        stopAudio();
                      }}
                    />
                  )}
                  {type === TYPE.REORDER && (
                    <Reorder
                      item={item}
                      dispatch={dispatch}
                      isFinish={isFinish}
                      user_answer={user_answer}
                    />
                  )}
                  {type === TYPE.MATCHING && (
                    <Matching
                      item={item}
                      dispatch={dispatch}
                      pSound={(res) => (sound = res)}
                      isFinish={isFinish}
                      isResult={isResult}
                      user_answer={user_answer}
                    />
                  )}
                  {type === TYPE.FILL_IN_THE_BLANK && (
                    <FillInTheBlank
                      item={item}
                      dispatch={dispatch}
                      isFinish={isFinish}
                      user_answer={user_answer}
                      audio={() => {
                        stopAudio();
                      }}
                    />
                  )}
                  {type === TYPE.FILL_IN_THE_BLANK_WRITING && (
                    <FillInTheBlankWiring
                      item={item}
                      dispatch={dispatch}
                      isFinish={isFinish}
                      user_answer={user_answer}
                      audio={() => {
                        stopAudio();
                      }}
                    />
                  )}
                  {type === TYPE.AUDIO_RECORDING && (
                    <AudioRecording
                      item={item}
                      onPauseAudio={() => {
                        setChoose(true);
                        stopAudio();
                      }}
                      user_answer={user_answer}
                      isFinish={isFinish}
                    />
                  )}
                </View>
              </View>
            </View>
          );
        })}
      </Swiper>
    </View>
  );
};

==> react-native-hotqa-sdk-0.2.0.tgz: package/src/pages/start/view/index.js <==
import React, {useState, useRef, useEffect} from 'react';
import {TouchableOpacity, View, Text, Image, Dimensions} from 'react-native';
import Swiper from '../../../components/swiper';
import Icon from 'react-native-vector-icons/Ionicons';
import {createSelector} from 'reselect';
import {useDispatch, useSelector} from 'react-redux';
import Slider from '@react-native-community/slider';

import SingleChoiceAudio from '../../../components/single-choice-audio';
import SingleChoice from '../../../components/single-choice';
import MultipleChoice from '../../../components/multiple-choice';
import ImageChoice from '../../../components/image-choice';
import Writing from '../../../components/writing';
import AudioRecording from '../../../components/audio-recording';
import Reorder from '../../../components/reorder/index2';
import Matching from '../../../components/matching';
import FillInTheBlank from '../../../components/fill-in-the-blank';
import FillInTheBlankWiring from '../../../components/fill-in-the-blank-writing';

import * as TYPE from '../../../common/constant';
import {onConfigurations} from '../../../common/helper';
import HTMLView from 'react-native-htmlview';
import {styles} from './style';
import ic_play from '../../../assets/images/ic_play.png';
import ic_pause from '../../../assets/images/ic_pause.png';
import song_nhac from '../../../assets/images/song_nhac.png';
import Sound from 'react-native-sound';
import {color_blue} from '../../../assets/colors';
import Player from '../../../components/player';
import {REQUEST_ANSWER_FROM_RESULT} from '../../../redux/actions';

const width = Dimensions.get('screen').width;

Icon.loadFont();
const getData = createSelector(
  (state) => state.reducersAccepted,
  (reducersAccepted) => reducersAccepted,
);

const getKey = createSelector(
  (state) => state.reducerComponents,
  (reducerComponents) => reducerComponents,
);
const getUserAnswer = createSelector(
  (state) => state.submitData,
  (submitData) => submitData,
);
export default (props) => {
  const {navigation, route} = props;
  const dispatch = useDispatch();
  // const player = useRef(null);
  // const [sound, setSound] = useState(null);
  // const [isPlay, setIsPlay] = useState(true);
  const [isChoose, setChoose] = useState(false);
  // const [muted, setMute] = useState(false);
  const {data, loading, error} = useSelector(getData);
  const {keyPage, isFinish, isResult} = useSelector(getKey);
  const {user_answer: answers} = useSelector(getUserAnswer);

  const arrAnswer = [];
  let questions;

  if (data) {
    const {sections} = data;
    const sectionsData = sections.map((e, index) => {
      const section_id = e.id;
      const section_tag = e.tag;
      const arr_question = e.questions || e.answers;
      const outline_s = e.outline;
      const extra = e.extra;
      const titleSection = e.title;
      const questionsData = arr_question.map((item) => {
        if (isResult) {
          item.id = item.question_id;
          item.title = item.question_title;
          item.type = item.question_type;
          item.configurations = item.question_configuration;
          item.extra = item.question_extra;
          item.tag = item.section_tag;

          const parseAnswer = JSON.parse(item.user_answer);
          if (parseAnswer.value !== '') {
            const itemAnswer = {
              question_id: item.question_id,
              type: item.question_type,
              user_answer: JSON.parse(item.user_answer),
              question_configuration: item.question_configuration,
            };
            arrAnswer.push(itemAnswer);
          }
        } else {
          const itemAnswer = {
            section_id: section_id,
            question_id: item.id,
            type: item.type,
            user_answer: {value: ''},
            tag: section_tag,
            question_configuration: JSON.parse(item.configurations),
          };
          arrAnswer.push(itemAnswer);
        }
        let configurations_type = '';
        const configurations = JSON.parse(item.configurations);
        if (Array.isArray(configurations) && configurations.length > 0) {
          configurations_type = configurations[0].data.type;
        }
        if (item.extra && JSON.parse(item.extra).length > 0) {
          return {
            ...item,
            section_id,
            section_tag,
            outline_s,
            configurations_type,
            titleSection,
          };
        }
        return {
          ...item,
          section_id,
          section_tag,
          outline_s,
          extra,
          configurations_type,
          titleSection,
        };
      });
      return questionsData;
    });
    questions = sectionsData.flat();
  }
  let limitMedia = 3;
  if (data && data.extra) {
    const parseExtra = JSON.parse(data.extra);
    if (parseExtra.media) {
      const {limit_play} = parseExtra.media;
      if (limit_play > 0) {
        limitMedia = limit_play;
      }
    }
  }
  const [limitPlay, setLimitPlay] = useState(limitMedia);
  // console.log(63, test_data.extra);

  let sound = null;
  let user_answer = answers;
  if (isResult) {
    user_answer = arrAnswer;
  }

  useEffect(() => {
    dispatch({type: REQUEST_ANSWER_FROM_RESULT, payload: {data: arrAnswer}});
  }, []);

  const onPauseAudio = (type) => {
    if (sound) {
      if (type === 'Record') {
        sound.stop();
      }
    }
  };

  Sound.setCategory('Playback', false);
  const playAudio = (item, index) => {
    if (keyPage === index && limitPlay > 0) {
      if (!sound) {
        sound = new Sound(item.value, null, (error) => {
          if (error) {
            return;
          }
          if (!isChoose) {
            sound.play(() => setLimitPlay(limitPlay - 1));
          }
        });
      }
    }
  };

  const stopAudio = () => {
    if (sound) {
      sound.stop();
    }
  };

  const viewTitle = (item, i) => {
    switch (item.type) {
      case 'image':
        return <Image source={{uri: item.value}} style={styles.imgQuestion} />;
      case 'text':
        // return <Text style={styles.outline}>{item.value}</Text>;
        return <HTMLView value={item.value} style={{marginBottom: 10}} />;
      case 'audio':
        playAudio(item, i);
        return (
          // <Player
          //   keyPage={keyPage}
          //   item={item}
          //   index={i}
          //   sound={sound}
          //   ref={player}
          // />
          <View style={styles.viewPlayer}>
            <TouchableOpacity
              onPress={() => {
                if (sound) {
                  // setChoose(false);
                  sound.play(() => setLimitPlay(limitPlay - 1));
                }
              }}>
              <Image source={ic_play} style={styles.iconPlayPause} />
            </TouchableOpacity>
            <Image
              source={song_nhac}
              style={{flex: 1, height: 50, opacity: 0.3}}
            />
          </View>
        );
    }
  };

  if (!data) {
    return <View />;
  }

  return (
    <View>
      <Swiper
        questions={questions}
        navigation={navigation}
        stopSound={() => {
          setChoose(false);
          setLimitPlay(limitMedia);
          stopAudio();
        }}
        keyPage={keyPage}
        route={route.params?.nav}>
        {questions.map((item, i) => {
          const extra =
            JSON.parse(item.extra) === null ? [] : JSON.parse(item.extra);
          const configurations = JSON.parse(item.configurations);

          const type = item.type;
          const {outline, outline_s} = item;
          // console.log(166, i, item);
          return (
            <View style={[styles.slide]} key={i}>
              {(outline_s ||
                outline ||
                (extra !== null && extra.length > 0)) && (
                <View style={styles.viewOutline}>
                  {outline_s ? (
                    <HTMLView
                      value={outline_s}
                      style={{marginHorizontal: 20}}
                    />
                  ) : null}
                  {outline ? (
                    <HTMLView value={outline} style={{marginHorizontal: 20}} />
                  ) : null}
                  {extra.length > 0 && (
                    <View style={styles.viewTitle}>
                      {/* {viewTitle(extra)} */}
                      {extra[0] && viewTitle(extra[0], i)}
                      {extra[1] && viewTitle(extra[1], i)}
                    </View>
                  )}
                </View>
              )}
              <View style={{flex: 0.6}}>
                <Text style={styles.title}>{item.title}</Text>
                <View style={{marginTop: 10}}>
                  {type === TYPE.SINGLE_CHOICE &&
                    item.configurations_type !== 'image' && (
                      <SingleChoice
                        data={configurations}
                        item_question={item}
                        user_answer={user_answer}
                        onPress={(items) => {
                          setChoose(true);
                          stopAudio();
                          onConfigurations(
                            {...item, items},
                            user_answer,
                            dispatch,
                          );
                        }}
                        keyPage={keyPage}
                        index={i}
                        isFinish={isFinish}
                      />
                    )}

                  {type === TYPE.MULTIPLE_CHOICE &&
                    item.configurations_type !== 'image' && (
                      <MultipleChoice
                        onPress={(items) => {
                          setChoose(true);
                          stopAudio();
                          onConfigurations(
                            {...item, items},
                            user_answer,
                            dispatch,
                          );
                        }}
                        data={configurations}
                        item_question={item}
                        user_answer={user_answer}
                        keyPage={keyPage}
                        index={i}
                        isFinish={isFinish}
                      />
                    )}

                  {item.configurations_type === 'image' && (
                    <ImageChoice
                      data={configurations}
                      type={type}
                      dispatch={dispatch}
                      user_answer={user_answer}
                      item_question={item}
                      keyPage={keyPage}
                      index={i}
                      isFinish={isFinish}
                      audio={() => {
                        setChoose(true);
                        stopAudio();
                      }}
                    />
                  )}
                  {type === TYPE.WRITING && (
                    <Writing
                      item={item}
                      user_answer={user_answer}
                      dispatch={dispatch}
                      isFinish={isFinish}
                      audio={() => {
                        stopAudio();
                      }}
                    />
                  )}
                  {type === TYPE.REORDER && (
                    <Reorder
                      item={item}
                      dispatch={dispatch}
                      isFinish={isFinish}
                      user_answer={user_answer}
                    />
                  )}
                  {type === TYPE.MATCHING && (
                    <Matching
                      item={item}
                      dispatch={dispatch}
                      pSound={(res) => (sound = res)}
                      isFinish={isFinish}
                      isResult={isResult}
                      user_answer={user_answer}
                    />
                  )}
                  {type === TYPE.FILL_IN_THE_BLANK && (
                    <FillInTheBlank
                      item={item}
                      dispatch={dispatch}
                      isFinish={isFinish}
                      user_answer={user_answer}
                      audio={() => {
                        stopAudio();
                      }}
                    />
                  )}
                  {type === TYPE.FILL_IN_THE_BLANK_WRITING && (
                    <FillInTheBlankWiring
                      item={item}
                      dispatch={dispatch}
                      isFinish={isFinish}
                      user_answer={user_answer}
                      audio={() => {
                        stopAudio();
                      }}
                    />
                  )}
                  {type === TYPE.AUDIO_RECORDING && (
                    <AudioRecording
                      item={item}
                      onPauseAudio={() => {
                        setChoose(true);
                        stopAudio();
                      }}
                      user_answer={user_answer}
                      isFinish={isFinish}
                    />
                  )}
                </View>
              </View>
            </View>
          );
        })}
      </Swiper>
    </View>
  );
};
